#ifndef MOVINGSTRING_H
#define MOVINGSTRING_H
#include <cstring>           // strlen(), strncpy()
#include <iostream>          // ostream, cout
#include <utility>           // swap, move

// ggf auskommentieren
#define STDMOVE          // return std::move() statt return bei op+(,), siehe  unten
#define TEST

#ifdef TEST
namespace {
int anzahlNew = 0;
}
#define TESTAUSGABE(x) std::cout << #x << ' ' << ++anzahlNew << std::endl;
#define TESTAUSGABE_NEUTRAL(x) std::cout << #x << std::endl;
#else
#define TESTAUSGABE(x)         /* nichts */
#define TESTAUSGABE_NEUTRAL(x) /* nichts */
#endif

class [[nodiscard]] StringTyp {
public:
  StringTyp(const char* s) 
    : len{static_cast<int>(std::strlen(s))}, capacity {len},
    start{new char[len + 1]}
  {
    TESTAUSGABE(StringTyp(const char* s) : new)
    std::strncpy(start, s, len+1);
  }

  StringTyp(const StringTyp& s) 
  : len{s.len}, capacity {len}, start{new char[len + 1]}
  {
    TESTAUSGABE(StringTyp(const StringTyp& s) : new)
    std::strncpy(start, s.start, len+1);
  }

  // neu
  StringTyp(StringTyp&& s)          // bewegender Konstruktor (resource stealing)
      : len{s.len}, capacity {len}, start{s.start}  // Speicher übernehmen
  { 
    TESTAUSGABE_NEUTRAL(StringTyp(const StringTyp&& s) : move)
    s.start = nullptr;          // nicht vergessen (Destruktor)
  }

  ~StringTyp() { delete[] start; }

  // geändert: Übergabe per const&, andernfalls Zweideutigkeit
  StringTyp& operator=(const StringTyp& rhs)
  {
    if(this == &rhs) {
      return *this;
    }
    if(capacity < rhs.len) {  // Platz reicht nicht
      auto tmp(rhs);          // new wird hier hochgezählt
      swap(tmp);
      TESTAUSGABE_NEUTRAL(= (const&)aufgerufen (mit swap()))
    }
    else {
      TESTAUSGABE_NEUTRAL(= (const&)aufgerufen (ohne swap()))
      len = rhs.len;
      std::strncpy(start, rhs.start, len+1); // +1 wg. Nullbyte    
    }
    return *this;
  }

  StringTyp& operator=(StringTyp&& rhs) noexcept  // bewegender Zuweisungsoperator
  { 
    if(this != &rhs) {
      TESTAUSGABE_NEUTRAL(= (&&)aufgerufen(move))
      swap(rhs);          // Ressourcen und Verwaltungsinformationen tauschen
    }
    return *this;
  }

  StringTyp& operator+=(const StringTyp& s)
  {
    char* temp = new char[len + s.len + 1];          // neuen Platz beschaffen
    TESTAUSGABE(operator+=(const StringTyp& s) : new)
    std::strncpy(temp, start, len);                  // Teil 1 kopieren
    std::strncpy(temp + len, s.start, s.len+1);      // Teil 2 kopieren
    delete[] start;                              // alten Platz freigeben
    len += s.len;                                // Verwaltungsinformation aktualisieren
    capacity = len;
    start = temp;
    return *this;
  }

  [[nodiscard]] const char* c_str() const { return start; }  // C-String zurückgeben

  // Eine Optimierung, falls capacity ausreicht, fehlt noch.
  StringTyp& insert(int pos, const StringTyp& s)       // s vor Position pos einfügen
  { 
    char* temp = new char[len + s.len + 1];            // neuen Platz beschaffen
    TESTAUSGABE(insert(int pos, const StringTyp& s) : new)
    std::strncpy(temp, start, pos);                        // die ersten pos Zeichen kopieren
    std::strncpy(temp + pos, s.start, s.len);              // s einfügen
    std::strncpy(temp + pos + s.len, start + pos, s.len+1);     // Rest kopieren
    delete[] start;                                        // alten Platz freigeben
    start = temp;                                          // Verwaltungsinformation aktualisieren
    len += s.len;
    capacity = len;
    return *this;
  }
  void swap(StringTyp& m) noexcept
  {
    std::swap(len, m.len);
    std::swap(capacity, m.capacity);
    std::swap(start, m.start);
  }

private:
  int len;                  // Länge
  int capacity;             // Kapazität
  char* start;              // Zeiger auf den Anfang
};

inline std::ostream& operator<<(std::ostream& os, const StringTyp& s)
{
  os << s.c_str();
  return os;
}

inline auto operator+(const StringTyp& x, const StringTyp& y)
{
  TESTAUSGABE_NEUTRAL(+(const&, const&)aufgerufen)
  StringTyp temp(x);
  temp += y;
  return temp;
}

#ifdef STDMOVE
// neu
// dito, wenn temporäre Objekte beteiligt sind.
inline auto operator+(StringTyp&& x, const StringTyp& y)
{
  TESTAUSGABE_NEUTRAL(Op2 : +(&&, const&)aufgerufen(move))
  return std::move(x += y);
}

inline auto operator+(const StringTyp& x, StringTyp&& y)
{
  TESTAUSGABE_NEUTRAL(Op3 : +(const&, &&)aufgerufen(move))
  // return std::move(StringTyp(x) += y);  besser, weil 1 new weniger:
  return std::move(y.insert(0, x));
}

inline auto operator+(StringTyp&& x, StringTyp&& y)
{
  TESTAUSGABE_NEUTRAL(Op4 : +(&&, &&)aufgerufen(move))
  return std::move(x += y);
}
#else
// neu
// dito, wenn temporäre Objekte beteiligt sind.
inline auto operator+(StringTyp&& x, const StringTyp& y)
{
  TESTAUSGABE_NEUTRAL(+(&&, const&)aufgerufen(move))
  return x += y;
}

inline auto operator+(const StringTyp& x, StringTyp&& y)
{
  TESTAUSGABE_NEUTRAL(+(const&, &&)aufgerufen(move))
  // return  StringTyp(x) += y; besser, weil 1 new weniger:
  return y.insert(0, x);
}

inline auto operator+(StringTyp&& x, StringTyp&& y)
{
  TESTAUSGABE_NEUTRAL(+(&&, &&)aufgerufen(move))
  return x += y;
}

#endif

#endif          // movingstring.h
