// wie lambda2.cpp, aber mit Variante  mit verschachtelter Lambda-Funktion
#include <cmath>
#include <iostream>
#include <vector>

// Funktion zum Summieren. Tatsächlich hängen die Details von der übergebenen Funktion f ab. 
// Es wäre z.B. möglich, statt der Summe das Produkt aller Elemente zu berechnen.
template <class Container, typename T, typename Function>
T berechne(Container& cont, T anfangswert, Function f)
{
  T ergebnis{anfangswert};
  for (auto wert : cont) {
    //  f() gibt einen veränderten Wert von ergebnis zurück
    ergebnis = f(ergebnis, wert);
  }
  return ergebnis;
}

int main()
{
  std::vector<double> v{3.1, 0.2, 22.0, 3.5, 8.99};
  // Berechnung der Summe mit einer generischen Lambda-Funktion
  auto summe{berechne(v, 0.0, [](auto sum, auto w) { return sum + w; })};

  // Initialisierung des Mittelwerts mit summe/ssize(v) in der Lambda-Funktion
  // Vorbedingung: std::ssize(v) > 0
  auto varianz = berechne(v, 0.0,
                          [mittelwert = summe / std::ssize(v)](auto sum, auto w)
                          {
                            auto diff = w - mittelwert;
                            return sum + diff * diff;
                          }) / ssize(v);
  std::cout << "Standardabweichung = " << std::sqrt(varianz) << '\n';

  // Variante  mit verschachtelter Lambda-Funktion
  auto varianz2 =
      berechne(v, 0.0,
               [mittelwert = berechne(v, 0.0, [](auto sum, auto w) { return sum + w; }) /
                             std::ssize(v)](auto sum, auto w) 
               {
                 auto diff = w - mittelwert;
                 return sum + diff * diff;
               }) / ssize(v);
  std::cout << "Standardabweichung = " << std::sqrt(varianz2) << '\n';
}

