#include <cassert>
#include <future>
#include <iostream>
#include <numeric>                           // iota()
#include <vector>

[[nodiscard]] long long summe(const std::vector<int>& v, int von, int bis)
{
  long long ergebnis{0ll};
  for (int i = von; i < bis; ++i) {
    ergebnis += v[i];
  }
  return ergebnis;
}

[[nodiscard]] long long parallelSumme(const std::vector<int>& v)
{
  auto anzahlTasks {std::thread::hardware_concurrency()}; // Anz. der Prozessorkerne
  std::vector<std::future<long long>> tasks;
  // Berechnung der laenge eines einzelnen Bereichs:
  auto laenge = 1000 + ssize(v) / anzahlTasks;            // Minimum 1000
  auto start{0L};
  while (start < ssize(v)) {
    auto ende = start + laenge;
    if (ende > ssize(v)) {
      ende = ssize(v);
    }
    // Task für Teilbereich [start, ende) starten und future-Objekt speichern:
    tasks.push_back(std::async(std::launch::async, summe, v, start, ende));
    // nur zur Dokumentation der Aufteilung:
    std::cout << "Bereich " << tasks.size() << ": " << start << '-' << ende - 1
              << '\n';                       // -1, weil v[ende] nicht mehr addiert wird
    start += laenge;
  }
  // Zwischenergebnisse einsammeln:
  long long ergebnis{0LL};
  for (auto& f : tasks) {                    // auto = std::future<long long>
    ergebnis += f.get();
  }
  return ergebnis;
}

int main()
{
  std::vector<int> v(1'000'000);
  std::iota(v.begin(), v.end(), 1);          // v mit 1, 2, 3 ... belegen
  auto erg = parallelSumme(v);
  std::cout << "Die Summe ist " << erg << ".\n";
  long long n = ssize(v);
  assert(erg == n * (1 + n) / 2);            // Überprüfung mit Gaußformel
}

