#ifndef QTBEISPIEL_H
#define QTBEISPIEL_H
#include "Zeichenflaeche.h"
#include <QtWidgets/QDial>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpinBox>

class QtBeispiel : public QWidget {
  Q_OBJECT
public:
  QtBeispiel(QMainWindow* parent = 0)
     : QWidget(parent), zeichenflaeche(new Zeichenflaeche(this)),
       qdial(new QDial(this)), qspinbox(new QSpinBox(this))
  {
    qspinbox->setFont(QFont("Helvetica", 12, QFont::StyleNormal));
    qspinbox->setMinimum(1);
    qspinbox->setMaximum(100);
    qdial->setMaximum(qspinbox->maximum());
    qdial->setNotchesVisible(true);          // Marken sichtbar machen
    maxWertButton = new QPushButton(
               QString::number(qspinbox->maximum()) + "%", this);
    maxWertButton->setFont(QFont("Helvetica", 16, QFont::Normal));

    // Connect-Aufrufe:
    // qdial signalisiert Änderungen der Einstellung. Empfänger: zeichenflaeche,
    // deren Methode setRadius(int) aufgerufen wird.
    connect(qdial, &QDial::valueChanged, 
            zeichenflaeche, &Zeichenflaeche::setRadius);

    // qdial signalisiert Änderungen der Einstellung. Empfänger: qspinbox,
    //  deren Methode setValue(int) aufgerufen wird.
    connect(qdial, &QDial::valueChanged, qspinbox, &QSpinBox::setValue);

    //  qspinbox signalisiert Änderungen. Empfänger: qdial,
    // dessen Methode setValue(int) aufgerufen wird.
    // valueChanged ist überladen. QOverload<int>::of wählt valueChanged(int) aus.
    connect(qspinbox, QOverload<int>::of(&QSpinBox::valueChanged), 
            qdial, &QDial::setValue);

    connect(maxWertButton, &QPushButton::clicked, 
            this, &QtBeispiel::setMaxwert);

    QGridLayout* gridLayout = new QGridLayout(this);
    gridLayout->addWidget(maxWertButton, 0, 0);
    gridLayout->addWidget(qspinbox, 1, 0);
    gridLayout->addWidget(qdial, 2, 0);
    gridLayout->addWidget(zeichenflaeche, 0, 1, 3, 1);
    // Zeile, Spalte, erstreckt sich über 3 Zeilen und eine Spalte
    gridLayout->setColumnStretch(1, 10);     // Spalte 1 breiteren Raum einräumen
    setLayout(gridLayout);
    qdial->setValue(1);                      // Startwert
  }

  const Zeichenflaeche* getZeichenflaeche() const { return zeichenflaeche; }
  const auto maxradius() const { return qspinbox->maximum(); }
  
private slots:
  void setMaxwert() { qdial->setValue(qspinbox->maximum()); }

private:
  Zeichenflaeche* zeichenflaeche;
  QDial* qdial;
  QSpinBox* qspinbox;
  QPushButton* maxWertButton;
};
#endif

