#ifndef HTTPCONNECTION_H
#define HTTPCONNECTION_H
#include "AnfrageHandler.h"
#include "DateiHandler.h"
#include <array>
#include <iostream>

class HttpConnection {
public:
  HttpConnection(boost::asio::ip::tcp::acceptor& a, std::string v)
      : acceptor(a), socket(a.get_executor()), verzeichnis(std::move(v))
  { }

  void accept() { acceptor.accept(socket); }

  void operator()()
  {
    constexpr int puffergroesse{1024};
    std::array<char, puffergroesse + 1> daten {'\0'};
    // Anfrage lesen
    auto anzahl = socket.read_some(boost::asio::buffer(daten, puffergroesse));
    daten.at(anzahl) = '\0';
    std::cout << daten.data() << '\n';
    std::string header(daten.data());
    auto leerzeichen = header.find(' ');
    // Fehlermeldung bei falschem Protokoll
    if (header.substr(0, leerzeichen) != "GET") {
      HttpResponse httpResponse(400);
      httpResponse.addToHeader("Content-Type", "text/html");
      std::string msg = httpResponse.getHTMLMessage(
                                        "Protokoll nicht implementiert!");
      httpResponse.addToHeader("Content-Length", std::to_string(msg.length()));
      httpResponse.sendHeader(socket);
      socket.write_some(boost::asio::buffer(msg.c_str(), msg.length()));
      return;                                // Abbruch!
    }
    auto leerzeichen2 = header.find(' ', leerzeichen + 1);
    auto pfad = header.substr(leerzeichen + 1, leerzeichen2 - leerzeichen - 1);
    // Gibt es eine Anfrage oder ist eine Datei gewünscht?
    auto qpos = pfad.find('?');
    if (qpos != std::string::npos) {         // Anfrage
      AnfrageHandler ah(socket, pfad.substr(qpos + 1));
      ah.process();
    }
    else {                                   // Datei
      auto dateiname = verzeichnis + pfad;
      if (pfad == "/" || pfad == "/favicon.ico") {
        dateiname = verzeichnis + "/index.html";
      }
      DateiHandler dh(socket, dateiname);
      dh.process();
    }
    socket.close();
  }
private:
  boost::asio::ip::tcp::acceptor& acceptor;
  boost::asio::ip::tcp::socket socket;
  std::string verzeichnis;
};
#endif
