#ifndef DATENBANK_H
#define DATENBANK_H
#include <sqlite3.h>
#include <stdexcept>
#include <string>
#include <string_view>
#include <vector>
using Tabelle = std::vector<std::vector<std::string>>;

class Datenbank {
public:
  Datenbank(const char* dateiname) 
  {
    if (sqlite3_open(dateiname, &db) != SQLITE_OK) {  // Fehler?
      auto fehlermeldung{sqlite3_errmsg(db)};
      sqlite3_close(db);
      throw std::runtime_error(fehlermeldung);
    }
  }

  ~Datenbank() { sqlite3_close(db); }         // Dekl. verhindert autom. move-Operationen
  Datenbank(const Datenbank&) = delete;             // Kopieren verbieten
  Datenbank& operator=(const Datenbank&) = delete;  // Zuweisen verbieten

  [[nodiscard]] Tabelle execute(std::string_view sql) const
  {
    return Auswerter().auswerten(sql, db);
  }

private:
  struct Auswerter {                                  // innere Klasse
    Auswerter() = default;
    ~Auswerter() { sqlite3_finalize(statement);  }
    Auswerter(const Auswerter&) = delete;
    Auswerter& operator=(const Auswerter&) = delete;

    Tabelle auswerten(std::string_view sql, sqlite3* database) 
    {
      if( sqlite3_prepare_v2(database, sql.data(), -1,
                             &statement, nullptr) != SQLITE_OK) {
        throw std::runtime_error("sqlite3_prepare fehlgeschlagen");
      }
      Tabelle ergebnis;
      int spalten {sqlite3_column_count(statement)};
      while (sqlite3_step(statement) == SQLITE_ROW) {
        std::vector<std::string> zeile;
        for (int s = 0; s < spalten; ++s) {
          auto str = reinterpret_cast<const char*>(
                      sqlite3_column_text(statement, s));  // liefert unsigned char*
          zeile.push_back(str ? str : std::string(""));
        }
        ergebnis.push_back(zeile);
      }
    return ergebnis;
    }

    sqlite3_stmt* statement {nullptr};
  };                                                // Ende der inneren Klasse

  sqlite3* db {nullptr};
};
#endif

