#ifndef LOCALEUTILS_H
#define LOCALEUTILS_H
#include <algorithm>
#include <iostream>
#include <locale>
#include <string>

template <typename charT>
void klein(std::basic_string<charT>& s, const std::locale& loc = std::locale())
{
  std::use_facet<std::ctype<charT>>(loc).tolower(s.data(), s.data() + s.length());
}

template <typename charT>
void gross(std::basic_string<charT>& s, const std::locale& loc = std::locale())
{
  std::use_facet<std::ctype<charT>>(loc).toupper(s.data(), s.data() + s.length());
}

template <typename charT>
struct EqualIgnoreCase {
  EqualIgnoreCase(const std::locale& lo = std::locale()) 
    : loc(lo) 
  {}
  [[nodiscard]] bool operator()(charT c1, charT c2) const
  { 
    const auto& facet {std::use_facet<std::ctype<charT>>(loc)};
    return facet.toupper(c1) == facet.toupper(c2); 
  }
  const std::locale& loc;
};

// nicht effizient
template <typename charT>
[[nodiscard]]
auto findeSuchstringIgnoreCase2(std::basic_string<charT> text,    // temporäre Kopie
                                std::basic_string<charT> such,    // temporäre Kopie
                                const std::locale& loc = std::locale())
{
  gross(text, loc);
  gross(such, loc);
  return text.find(such);
}

template <typename charT>
[[nodiscard]] 
auto findeSuchstringIgnoreCase(const std::basic_string<charT>& text,
                               const std::basic_string<charT>& such,
                               const std::locale& loc = std::locale())
{
  auto iter = std::search(text.begin(), text.end(), such.begin(), such.end(),
                          EqualIgnoreCase<charT>(loc));
  return iter == text.end() ? std::basic_string<charT>::npos : (iter - text.begin());
}
#endif
