#include <fstream>
#include <iostream>
#include <map>
#include <regex>
#include <string>
#include <string_view>

namespace {                                  // anonymer Namespace
[[nodiscard]] std::string ersetzeInDatei(const std::regex& gesucht,
                                         const std::string& ersatz,
                                         const std::string& dateiname)
{
  std::ifstream quelle(dateiname, std::ios::binary);
  if (!quelle.good()) {
    throw std::ios::failure("Dateifehler");
  }
  std::string alles("");
  while (quelle.good()) {
    char c = static_cast<char>(quelle.get());
    if (!quelle.fail()) {
      alles += c;
    }
  }
  return std::regex_replace(alles, gesucht, ersatz);
}
}                                            // anonymer Namespace
using namespace std;

int main(int argc, char* argv[])
{
  constexpr string_view gebrauch(
      "Gebrauch: ersetze.exe [-i] \"regex\" \"ersatz\" \"dateiname\"");
  auto syntaxoption{regex_constants::ECMAScript};  // Voreinstellung
  int start{1};
  if (5 == argc) {                           // Ist evtl. die Option -i gesetzt?
    string_view option(argv[1]);
    if (option != "-i") {
      cerr << "Falsche Option: " << gebrauch << '\n';
      return 1;                              // EXIT
    }
    ++start;
    syntaxoption |= regex_constants::icase;  // Klein-/Großschreibung ignorieren
  }
  else if (4 != argc) {
    cout << gebrauch << '\n';
    return 2;                                // EXIT
  }
  // Datei durchsuchen
  try {
    regex gesucht(argv[start], syntaxoption);
    string ersatz(argv[start + 1]);
    string ergebnis = ersetzeInDatei(gesucht, ersatz, argv[start + 2]);
    cout << ergebnis;
  }
  catch (regex_error& re) {
    cerr << "Regex-Fehler: " << re.what() << '\n';
  }
  catch (ios::failure& e) {
    cerr << e.what() << '\n';
  }
}
