#ifndef ARRAY2D_H
#define ARRAY2D_H
#include <initializer_list>
#include <iostream>
#include <stdexcept>
#include <vector>

template <typename T>
class Array2d {
public:
  Array2d(int z, int s, const T& wert = T())
  : zeilen_{z}, spalten_{s}, daten{std::vector<T>(z * s, wert)}
  { }

  // Sequenzkonstruktor
  Array2d(std::initializer_list<std::initializer_list<T>> liste)
  : Array2d(liste.size(), liste.begin()->size())
  {
    int index{0};
    for (const auto& zeileninitialisierer : liste) {
      for (const auto& element : zeileninitialisierer) {
        daten[index++] = element;
      }
    }
  }

  auto zeilen() const { return zeilen_; }
  auto spalten() const { return spalten_; }

  void check_indizes(int z, int s) const
  {
    const std::string datei {__FILE__};
    if (z < 0 || z >= zeilen_) {
      throw std::range_error(datei +": z=" + std::to_string(z) + " ungültig!");
    }
    if (s < 0 || s >= spalten_) {
      throw std::range_error(datei +": s=" + std::to_string(s) + " ungültig!");
    }
  }

  const auto& operator()(int z, int s) const 
  { 
#ifdef INDEXCHECK
    check_indizes(z, s);
#endif
    return daten[+ z * spalten_ + s]; 
  }
  auto& operator()(int z, int s) 
  {
#ifdef INDEXCHECK
    check_indizes(z, s);
#endif
    return daten[+ z * spalten_ + s]; 
  }

private:
  int zeilen_ = 0;
  int spalten_ = 0;
  std::vector<T> daten;
};

template <typename T>
void print(const Array2d<T>& a, 
           std::ostream& ausgabe = std::cout) 
{
  for (int z = 0; z < a.zeilen(); ++z) {
    for (int s = 0; s < a.spalten(); ++s) {
      ausgabe << a(z, s) << " ";
    }
    ausgabe << '\n';
  }
  ausgabe << '\n';
}
#endif
