#include <iostream>
#include <map>
#include <string_view>

enum class Sortiermodus { aufsteigend, absteigend };

class Compare {
public:
  Compare(Sortiermodus m = Sortiermodus::absteigend) 
  : sortiermodus{m} 
  {}

  template <typename T>
  [[nodiscard]] bool operator()(const T& t1, const T& t2) const
  {
    return sortiermodus == Sortiermodus::aufsteigend ? t1 < t2 : t2 < t1;
  }

private:
  Sortiermodus sortiermodus;
};

int main()
{
  // ohne Compare-Klasse, d.h. less<string_view> (aufsteigend)
  std::map<std::string_view, int> m0{{"Lena", 3}, {"Hanna", 9}};
  for (const auto& paar : m0) {
    std::cout << paar.first << "   " << paar.second << '\n';
  }

  // mit Compare-Klasse, aber ohne Compare-Objekt, d.h. Sortiermodus::absteigend
  // (im Konstruktor voreingestellt)
  std::map<std::string_view, int, Compare> m1{{"Lena", 3}, {"Hanna", 9}};
  for (const auto& paar : m1) {
    std::cout << paar.first << "   " << paar.second << '\n';
  }

  // mit Compare-Klasse und Compare-Objekt mit Argument
  std::map<std::string_view, int, Compare> m2({{"Lena", 3}, {"Hanna", 9}},
                                         Compare(Sortiermodus::aufsteigend));
  for (const auto& paar : m2) {
    std::cout << paar.first << "   " << paar.second << '\n';
  }
}

