#ifndef ARRAY2D_H
#define ARRAY2D_H
#include <cassert>
#include <iostream>

template <typename T>
class Array2d {
public:
  Array2d(int z, int s)
  : zeilen{z}, spalten{s}, ptr{new T[z * s]} {}

  Array2d(int z, int s, const T& wert) 
  : Array2d(z, s)                           // an obigen Konstruktor delegieren
  {
    init(wert);
  }

  Array2d(const Array2d& a) 
  : Array2d(a.zeilen, a.spalten)            // an obigen Konstruktor delegieren
  {
    int anzahl = zeilen * spalten;
    for (int i = 0; i < anzahl; ++i) {
      ptr[i] = a.ptr[i];
    }
  }

  ~Array2d() { delete[] ptr; }

  Array2d& operator=(const Array2d& arr) = delete;  // (noch) verbieten

  auto& assign(Array2d tmp)
  {
    swap(tmp);
    return *this;
  }

  [[nodiscard]] auto getZeilen() const { return zeilen; }
  [[nodiscard]] auto getSpalten() const { return spalten; }

  void init(const T& wert)                  // Alle Elemente mit wert initialisieren
  {
    int anzahl = zeilen * spalten;
    for (int i = 0; i < anzahl; ++i) {
      ptr[i] = wert;
    }
  }

  const T& at(int z, int s) const           // für lesenden Zugriff
  {
    assert(z < zeilen && s < spalten);
    return ptr[z * spalten + s];
  }

  T& at(int z, int s)                       // für verändernden Zugriff
  {
    assert(z < zeilen && s < spalten);
    return ptr[z * spalten + s];
  }

  void swap(Array2d& rhs)            // Daten von *this mit denen von rhs vertauschen
  {
    int temp = zeilen;
    zeilen = rhs.zeilen;
    rhs.zeilen = temp;
    temp = spalten;
    spalten = rhs.spalten;
    rhs.spalten = temp;
    T* tempPtr = ptr;
    ptr = rhs.ptr;
    rhs.ptr = tempPtr;
  }
private:
  int zeilen;
  int spalten;
  T* ptr;
};

template <typename T>
void printArray(const Array2d<T>& a)                 // Freie Funktion zur Ausgabe
{
  for (int z = 0; z < a.getZeilen(); ++z) {
    for (int s = 0; s < a.getSpalten(); ++s) {
      std::cout << a.at(z, s) << " ";
    }
    std::cout << '\n';
  }
}
#endif

