#ifndef ARRAY2D_H
#define ARRAY2D_H
#include <cassert>
#include <iostream>

template <typename T>
class Array2d {
public:
  Array2d(int z, int s)
  : zeilen{z}, spalten{s}, kapazitaet{z * s}, ptr{new T[z * s]} {}

  Array2d(int z, int s, const T& wert) 
  : Array2d(z, s)                           // an obigen Konstruktor delegieren
  {
    init(wert);
  }

  Array2d(const Array2d& a) 
  : Array2d(a.zeilen, a.spalten)            // an obigen Konstruktor delegieren
  {
    int anzahl = zeilen * spalten;
    for (int i = 0; i < anzahl; ++i) {
      ptr[i] = a.ptr[i];
    }
  }

  ~Array2d() { delete[] ptr; }

  Array2d& operator=(const Array2d& arr) = delete;  // (noch) verbieten

/*
  auto& assign(Array2d tmp)
  {
    swap(tmp);
    return *this;
  }
*/

auto& assign(const Array2d& rhs)
{
  if(this == &rhs) {                // Selbstzuweisung berücksichtigen
    return *this;    
  }
  zeilen = rhs.getZeilen();         // Verwaltungsinformation aktualisieren
  spalten = rhs.getSpalten();
  auto platzbedarf = zeilen * spalten;
  if (kapazitaet >= platzbedarf) { // Platz genug 
    for (int k = 0; k < platzbedarf; ++k) {
        ptr[k] = rhs.ptr[k];
    }
  }
  else {                              // Platz reicht nicht
    auto tmp(rhs);
    swap(tmp);
  }
  return *this;
}

  auto getZeilen() const { return zeilen; }

  auto getSpalten() const { return spalten; }

  void init(const T& wert)                  // Alle Elemente mit wert initialisieren
  {
    int anzahl = zeilen * spalten;
    for (int i = 0; i < anzahl; ++i) {
      ptr[i] = wert;
    }
  }

  const T& at(int z, int s) const           // für lesenden Zugriff
  {
    assert(z < zeilen && s < spalten);
    return ptr[z * spalten + s];
  }

  T& at(int z, int s)                       // für verändernden Zugriff
  {
    assert(z < zeilen && s < spalten);
    return ptr[z * spalten + s];
  }

  void swap(Array2d& rhs)            // Daten von *this mit denen von rhs vertauschen
  {
    int temp = zeilen;
    zeilen = rhs.zeilen;
    rhs.zeilen = temp;
    temp = spalten;
    spalten = rhs.spalten;
    rhs.spalten = temp;
    T* tempPtr = ptr;
    ptr = rhs.ptr;
    rhs.ptr = tempPtr;
  }

private:
  int zeilen;
  int spalten;
  int kapazitaet;
  T* ptr;
};

// Vergleich
template <typename T>
bool istGleich(const Array2d<T>& a, const Array2d<T>& b)
{
  if(a.getZeilen() != b.getZeilen()) return false;
  if(a.getSpalten() != b.getSpalten()) return false;
  for (int z = 0; z < a.getZeilen(); ++z) {
    for (int s = 0; s < a.getSpalten(); ++s) {
      if(a.at(z, s) != b.at(z, s)) {
        return false;
      }
    }
  }
  return true;
}

// Freie Funktion zur Ausgabe
template <typename T>
void printArray(const Array2d<T>& a)
{
  for (int z = 0; z < a.getZeilen(); ++z) {
    for (int s = 0; s < a.getSpalten(); ++s) {
      std::cout << a.at(z, s) << " ";
    }
    std::cout << '\n';
  }
}
#endif

