#ifndef ZAEHLENDESOBJEKT_H
#define ZAEHLENDESOBJEKT_H
#include <cstddef>
#include <iostream>
#include <set>

class ZaehlendesObjekt {
public:
  ZaehlendesObjekt()
  {
    ++gesamt;
    //  istHeapObjekt ist true, falls im Set enthalten
    istHeapObjekt = (1 == objekte.count(this)); 
    std::cout << (istHeapObjekt ? "Heap" : "Stack") 
              << "-Objekt " << this << " erzeugt.\n";
  }
  virtual ~ZaehlendesObjekt()
  {
    --gesamt;
    std::cout << (istHeapObjekt ? "Heap" : "Stack") 
              << "-Objekt " << this << " zerstört.\n";
  }

  ZaehlendesObjekt(const ZaehlendesObjekt&) = delete;
  ZaehlendesObjekt& operator=(const ZaehlendesObjekt&) = delete;

  static void status()
  {
    std::cout << "Es gibt " << gesamt << " Objekt(e), davon " 
              << objekte.size() << " Heap-Objekt(e)\n";
  }

  [[nodiscard]] static void* operator new(std::size_t size)
  {
    void* ptr{::operator new(size)};
    objekte.insert(ptr);
    return ptr;
  }

  static void operator delete(void* ptr)
  {
    objekte.erase(ptr);
    ::operator delete(ptr);
  }
private:
  bool istHeapObjekt;
  static std::set<void*> objekte;
  static int gesamt;
};
#endif

