#ifndef SMARTPOINTER_H
#define SMARTPOINTER_H
#include "NullPointerException.h"

template <typename T> 
class SmartPointer final {                   // Vererbung ist nicht gewollt
public:
  explicit SmartPointer(T* p = nullptr);
  ~SmartPointer();
  void operator=(const SmartPointer&) = delete;          // Zuweisung verbieten
  SmartPointer(const SmartPointer&) = delete;   // Kopierkonstruktor verbieten
  [[nodiscard]] T* operator->() const;
  [[nodiscard]] T& operator*() const;
  [[nodiscard]] explicit operator bool() const;
private:
  T* zeigerAufObjekt;
  void check() const;          // Prüfung auf nicht-Null
};

template <typename T>
SmartPointer<T>::SmartPointer(T* p) : zeigerAufObjekt{p} {}

template <typename T>
void SmartPointer<T>::check() const
{
  if (!zeigerAufObjekt) {
    throw NullPointerException();
  }
}

template <typename T>
SmartPointer<T>::~SmartPointer() { delete zeigerAufObjekt; }

template <typename T>
T* SmartPointer<T>::operator->() const
{
  check();
  return zeigerAufObjekt;
}

template <typename T>
T& SmartPointer<T>::operator*() const
{
  check();
  return *zeigerAufObjekt;
}

template <typename T>
SmartPointer<T>::operator bool() const { return bool(zeigerAufObjekt); }

#endif
