#include "SmartPointer.h"
#include <iostream>
using namespace std;

class A {
public:
  A() = default;
  virtual ~A()            { cout << "A::Destruktor\n"; }
  A(const A&) = delete;
  A& operator=(const A&) = delete;
  virtual void hi() const { cout << "hier ist A::hi()\n"; }
};

class B : public A {
public:
  B() = default;
  ~B() override             { cout << "B::Destruktor\n"; }
  B(const B&) = delete;
  B& operator=(const B&) = delete;
  void hi() const override  { cout << "hier ist B::hi()\n"; }
};

// Übergabe per Wert ist hier nicht möglich, wohl aber per Referenz:
template <class T> void perReferenz(const SmartPointer<T>& p)
{
  cout << "Aufruf: perReferenz(const SmartPointer<T>&):";
  p->hi();          // (p.operator->())->hi();
}

int main()
{
  cout << "Zeiger auf dynamische Objekte:\n";
  cout << "Konstruktoraufruf\n";
  SmartPointer<A> spA(new A);
  cout << "Operator ->\n";
  spA->hi();
  cout << "Operator *\n";
  (*spA).hi();
  cout << "Polymorphismus:\n";
  SmartPointer<A> spAB(new B);          // zeigt auf B-Objekt
  spAB->hi();                           // B::hi()
  // Parameterübergabe eines SmartPointer
  perReferenz(spAB);

  // Die  Wirkung der Sicherungsmaßnahmen im Vergleich zu einfachen
  // C-Zeigern zeigen die folgenden Zeilen:
  SmartPointer<B> spUndef;
  try {
    if (!spUndef)                           // = if (!(spUndef.operator bool()))
      cout << "undefinierter Zeiger:\n";
    // Zugriff auf nicht-initialisierten Zeiger bewirkt Laufzeitfehler:
    spUndef->hi();                          // Laufzeitfehler!
    (*spUndef).hi();                        // Laufzeitfehler!
  }
  catch (const NullPointerException& ex) {
    cerr << "Laufzeitfehler: " << ex.what() << '\n';
  }
  //  Die folgenden Anweisungen bewirken Fehlermeldungen des Compilers
  // SmartPointer<B> spTyp(new A);     //  Fehler! Ein B ist kein A!
  // SmartPointer<A> spY = spA;        // Kopie nicht möglich!
  SmartPointer<A> spA1;
  // spA1 = spA;                       // Zuweisung nicht möglich!
}

