#include <array>
#include <cassert>
#include <fstream>
#include <iostream>
#include <string>
#include <source_location>
#include <system_error>

template <typename T, auto N>
void writeArray(const std::string& filename, const std::array<T, N>& a)
{
  std::ofstream dest(filename, std::ios::binary);
  if (!dest) {
    const auto& wo {std::source_location::current()};
    throw std::ios::failure(std::string(wo.file_name()) + ", Zeile " 
                             + std::to_string(wo.line()));
  }
  dest.write(reinterpret_cast<const char*>(a.data()), a.size() * sizeof(T));
}

template <typename T, auto N>
void readArray(const std::string& filename, std::array<T, N>& a)
{
  std::ifstream source(filename, std::ios::binary);
  if (!source) {
    const auto& wo {std::source_location::current()};
    throw std::ios::failure(std::string(wo.file_name()) + ", Zeile " 
                             + std::to_string(wo.line()));
  }
  source.read(reinterpret_cast<char*>(a.data()), a.size() * sizeof(T));
}

int main()
{
  try {
    const std::string dateiname("binaerdaten.bin");
    std::array a1{1.2, 3.4, 5.678, 9.01234, 100.83653};
    writeArray(dateiname, a1);          // a1 schreiben
    decltype(a1) a2{};                  // enthält 0.0
    readArray(dateiname, a2);           // a2 lesen
    assert(a1 == a2);                   // auf Gleichheit prüfen

    // funktioniert auch für Matrizen, z.B.
    const std::string dateiname1("binaerdatenMatrix.bin");
    std::array<std::array<double, 3>, 2> am1          // 2-dimensionale Matrix
      {{ {1.1, 2.2, 3.3},
         {4.4, 5.5, 6.6} }};
    writeArray(dateiname1, am1);          // am1 schreiben
    decltype(am1) am2{};                  // enthält 0.0
    readArray(dateiname1, am2);           // am2 lesen
    assert(am1 == am2);                   // auf Gleichheit prüfen
  }
  catch(const std::ios::failure& exc) {
    std::cerr << exc.what() << '\n';
  }
}
