//  Listen-Template für einfach-verkettete Liste
// T ist ein Platzhalter für den Datentyp eines Listenelements

/*******  Die Lösungen zu den Übungsaufgaben befinden
sich am Ende  **/

#ifndef LISTE_H
#define LISTE_H
#include <cstddef>
#include <iterator>

template <typename T>
class Liste {
public:

  // push_front() erzeugt ein neues Listenelement und
  // fügt es am Anfang der Liste ein:
  void push_front(const T& wert) { anfang = new ListElement(wert, anfang); }

private:
  struct ListElement {
    T daten;
    ListElement* naechstes{nullptr};
    ListElement(const T& wert, ListElement* p) : daten{wert}, naechstes{p} {}
  };
  ListElement* anfang{nullptr};

public:
  class Iterator {
    friend class Liste;

  public:
    // von find() geforderte Schnittstelle:
    using iterator_category = std::forward_iterator_tag;
    using value_type = T;
    using pointer = T*;
    using reference = T&;
    using difference_type = std::ptrdiff_t;

    Iterator(ListElement* init = nullptr) : aktuellesElement(init) {}

     [[nodiscard]] T& operator*()                          // Dereferenzierung
    {
      return aktuellesElement->daten;
    }

    [[nodiscard]] const T& operator*() const              // Dereferenzierung
    {
      return aktuellesElement->daten;
    }

    Iterator& operator++()                                // Präfix
    {
      if (aktuellesElement)          //  noch nicht am Ende angelangt?
        aktuellesElement = aktuellesElement->naechstes;
      return *this;
    }

    [[nodiscard]] Iterator operator++(int)                // Postfix
    { 
      Iterator temp = *this;
      ++*this;
      return temp;
    }

    [[nodiscard]] bool operator==(const Iterator& x) const { return aktuellesElement == x.aktuellesElement; }

  private:
    ListElement* aktuellesElement;          // Verweis auf aktuelles Element
  };                                        // Iterator
  //   Einige Methoden der Klasse Liste benutzen die Klasse
  //   Iterator (siehe unten):
  [[nodiscard]] Iterator begin() const { return Iterator(anfang); }
  [[nodiscard]] Iterator end() const { return Iterator(); }

  Liste() = default;

  Liste(const Liste& liste)                 // Kopierkonstruktor
  {
    if (liste.anfang) {
      Iterator i{liste.begin()};
      push_front(*i++);          // erstes Element anlegen, anfang ist danach != nullptr
      ListElement* letztes{anfang};
      while (i != liste.end()) {
        // Elemente am Ende einfügen, damit die Reihenfolge erhalten bleibt
        letztes->naechstes = new ListElement(*i++, nullptr);
        letztes = letztes->naechstes;
      }
    }
  }

  Liste& operator=(Liste temp)              // Zuweisungsoperator
  {
    std::swap(temp.anfang, anfang);
    return *this;
  }

  ~Liste()
  {          // Destruktor
    clear();
  }

  Iterator erase(Iterator p)
  {
    if (empty()) {
      return Iterator();                    // leere Liste
    }
    ListElement* zuLoeschen{p.aktuellesElement};
    // Vorgänger suchen
    ListElement* vorgaenger{anfang};
    if (zuLoeschen != anfang) {
      while (vorgaenger->naechstes != zuLoeschen) {
        vorgaenger = vorgaenger->naechstes;
      }
      // Zeiger verbiegen
      vorgaenger->naechstes = zuLoeschen->naechstes;
    }
    else {                                     // am Anfang löschen
      anfang = zuLoeschen->naechstes;          // Zeiger verbiegen
    }
    Iterator nachfolger{++p};
    delete zuLoeschen;
    return nachfolger;
  }

  void pop_front() { erase(begin()); }

  void clear()
  {
    while (!empty()) {
      pop_front();
    }
  }

  bool empty() const { return anfang == nullptr; }

  [[nodiscard]] int size() const
  {
    int anzahl{0};
    ListElement* temp{anfang};
    while (temp) {
      ++anzahl;
      temp = temp->naechstes;
    }
    return anzahl;
  }

  /*******  Ende der Lösungen  zu den Übungsaufgaben *************/
};

#endif

