#ifndef ABLAGE_H
#define ABLAGE_H
#include <condition_variable>
#include <mutex>
#include <thread>
#include <vector>
inline std::mutex ausgabeMutex;

class Ablage {
public:
  Ablage(int platz)
    : inhalt(std::vector<int>(platz))
  {} 

  [[nodiscard]] int get()
  {
    std::unique_lock<std::mutex> lock(monitorMutex);
    cond.wait(lock, [&,this]{ return anzahl > 0 || beendet; });
   --anzahl;
    cond.notify_all();
    lesePos = (lesePos + 1) % std::ssize(inhalt);
    return inhalt[lesePos];                  // ohne Bedeutung, falls beendet
  }

  void put(int wert)
  {
    std::unique_lock<std::mutex> lock(monitorMutex);
    cond.wait(lock, [&,this]{ return anzahl < std::ssize(inhalt) || beendet; });
    inhalt[schreibPos] = wert;
    ++anzahl;
    schreibPos = (schreibPos + 1) % std::ssize(inhalt);
    cond.notify_all();
  }

  void beenden()                             // alle wait()-Aufrufe beenden
  {
    std::unique_lock<std::mutex> lock(monitorMutex);
    beendet = true;
    cond.notify_all();
  }

  [[nodiscard]] bool istBeendet() const { return beendet; }

private:
  bool beendet{false};
  std::vector<int> inhalt;
  int anzahl{0};
  // Aufbau als Ringpuffer (FIFO)
  long lesePos {-1z};                 // letzte gelesene Pos (nicht unsigned, s.o.), Startwert -1
  long schreibPos{0z};                        // nächste zu schreibende Pos
  std::mutex monitorMutex;
  std::condition_variable cond;
};
#endif

