#ifndef RATIONAL_H
#define RATIONAL_H
#include <cassert>
#include <iostream>

[[nodiscard]] constexpr long ggt(long x, long y)   // alternativ std::gcd() nehmen
{
  long rest = 0;
  while (y > 0) {
    rest = x % y;
    x = y;
    y = rest;
  }
  return x;
}

class [[nodiscard]] Rational {
public:
  constexpr Rational(long z = 0, long n = 1) : zaehler{z}, nenner{n} {}

  // Abfragen
  [[nodiscard]] constexpr long getZaehler() const { return zaehler; }
  [[nodiscard]] constexpr long getNenner() const { return nenner; }

  // arithmetische Methoden für +=, -=, *=, /=,
  // (werden später durch überladene Operatoren ergänzt)
  constexpr void add(Rational r)
  {
    zaehler = zaehler * r.nenner + r.zaehler * nenner;
    nenner = nenner * r.nenner;
    kuerzen();
  }
  constexpr void sub(Rational r)
  {
    zaehler = zaehler * r.nenner - r.zaehler * nenner;
    nenner = nenner * r.nenner;
    kuerzen();
  }

  constexpr void mult(Rational r)
  {
    zaehler = zaehler * r.zaehler;
    nenner = nenner * r.nenner;
    kuerzen();
  }

  constexpr void div(Rational r)
  {
    zaehler = zaehler * r.nenner;
    nenner = nenner * r.zaehler;
    kuerzen();
  }

  constexpr void kuerzen()
  {
    // Vorzeichen merken und Betrag bilden
    int sign = 1;
    if (zaehler < 0) {
      sign = -sign;
      zaehler = -zaehler;
    }
    if (nenner < 0) {
      sign = -sign;
      nenner = -nenner;
    }
    long teiler = ggt(zaehler, nenner);          // siehe oben
    zaehler = sign * zaehler / teiler;           // Vorzeichen restaurieren
    nenner = nenner / teiler;
  }

  // nicht-constexpr Methoden
  void set(long z, long n);
  void kehrwert();

private:
  long zaehler, nenner;
};

// globale Funktionen
constexpr Rational add(Rational a, Rational b)
{
  a.add(b);
  return a;
}

constexpr Rational sub(Rational a, Rational b)
{
  a.sub(b);
  return a;
}

constexpr Rational mult(Rational a, Rational b)
{
  a.mult(b);
  return a;
}

constexpr Rational div(Rational z, Rational n)
{
  z.div(n);
  return z;
}

void eingabe(Rational& r);
void ausgabe(Rational r, std::ostream& = std::cout);

#endif

