#ifndef ARRAY2D_H
#define ARRAY2D_H
#include <iostream>
#include <stdexcept>
#include <string>
#include <vector>

template <typename T>
class [[nodiscard]] Array2d {
public:
  Array2d(int z, int s, const T& wert = T())
  : zeilen_{z}, spalten_{s}, daten{std::vector<T>(z * s, wert)}
  { }

  [[nodiscard]] auto zeilen() const { return zeilen_; }
  [[nodiscard]] auto spalten() const { return spalten_; }

  void check_indizes(int z, int s) const
  {
    if (z < 0 || z >= zeilen_) {
      throw std::range_error("Zeile " + std::to_string(z) + " ungültig!");
    }
    if (s < 0 || s >= spalten_) {
      throw std::range_error("Spalte" + std::to_string(s) + " ungültig!");
    }
  }
  // C++23
  const auto& operator[](int z, int s) const 
  { 
#ifdef INDEXCHECK
    check_indizes(z, s);
#endif
    return daten[z * spalten_ + s]; 
  }
  auto& operator[](int z, int s) 
  {
#ifdef INDEXCHECK
    check_indizes(z, s);
#endif
    return daten[z * spalten_ + s]; 
  }
  
  // mögliche Lösungen der Übungsaufgaben:
   void init(const T& wert) 
  {
    for (auto& element : daten) {
      element = wert;
    }
  }

  // Multiplikation mit einer Matrix b
  auto& operator*=(const Array2d<T>& b)
  {
    if (spalten() != b.zeilen()) {
      throw "Falsche Dimension in Matrix*= !\n";
    }
    Array2d<T> erg(zeilen(), b.spalten());
    for (int i = 0; i < zeilen(); ++i) {
      for (int j = 0; j < b.spalten(); ++j) {
        erg[i, j] = T(0);
        for (int k = 0; k < spalten(); ++k)
          erg[i, j] += operator[](i, k) * b[k, j];
      }
    }
    std::swap(*this, erg);                     // vertauschen
    return *this;
  }
  
  // Hilfsfunktion für den binären Gleichheitsoperator, s.u.
  const auto& get_daten() const { return daten; }

private:
  int zeilen_;
  int spalten_;
  std::vector<T> daten;
};

template <typename T>
void print(const Array2d<T>& a, std::ostream& ausgabe = std::cout) 
{
  for (int z = 0; z < a.zeilen(); ++z) {
    for (int s = 0; s < a.spalten(); ++s) {
      ausgabe << a[z, s] << " ";
    }
    ausgabe << '\n';
  }
  ausgabe << '\n';
}

// binäre Multiplikation
template <typename T> 
Array2d<T> operator*(Array2d<T> a, const Array2d<T>& b) { return a *= b; }

// Gleichheitsoperator
template <typename T> 
[[nodiscard]] bool operator==(const Array2d<T>& a, const Array2d<T>& b) 
{
  return a.get_daten() == b.get_daten();  // ruft vector-Vergleichsoperator
}
#endif
